@@ -1,5 +1,5 @@
 __author__ = "Johannes Köster"
-__copyright__ = "Copyright 2018-2019, Johannes Köster"
+__copyright__ = "Copyright 2021, Johannes Köster"
 __email__ = "johannes.koester@uni-due.de"
 __license__ = "MIT"
 
@@ -35,6 +35,7 @@ def cwl(
     use_singularity,
     bench_record,
     jobid,
+    runtime_sourcecache_path,
 ):
     """
     Load cwl from the given basedir + path and execute it.
@@ -52,7 +53,7 @@ def cwl(
         if not os.path.isabs(path):
             path = os.path.abspath(os.path.join(basedir, path))
         path = "file://" + path
-    path = format(path, stepout=1)
+    path = format(path, wildcards=wildcards)
     if path.startswith("file://"):
         sourceurl = "file:" + pathname2url(path[7:])
     else:
@@ -192,7 +193,7 @@ def job_to_cwl(job, dag, outputs, inputs):
 
 
 def dag_to_cwl(dag):
-    """Convert a given DAG to a CWL workflow, which is returned as JSON object."""
+    """Convert a given DAG to a CWL workflow, which is returned as a JSON object."""
     snakemake_cwl = {
         "class": "CommandLineTool",
         "id": "#snakemake-job",
@@ -220,7 +221,7 @@ def dag_to_cwl(dag):
                 "type": "File",
                 "default": {
                     "class": "File",
-                    "location": os.path.relpath(dag.workflow.snakefile),
+                    "location": os.path.relpath(dag.workflow.main_snakefile),
                 },
                 "inputBinding": {"prefix": "--snakefile"},
             },
