@@ -1,6 +1,6 @@
 __author__ = "Johannes Köster"
-__copyright__ = "Copyright 2015-2020, Johannes Köster"
-__email__ = "koester@jimmy.harvard.edu"
+__copyright__ = "Copyright 2021, Johannes Köster"
+__email__ = "johannes.koester@uni-due.de"
 __license__ = "MIT"
 
 import logging
@@ -21,6 +21,7 @@ from snakemake.exceptions import log_verbose_traceback
 from snakemake.exceptions import WorkflowError
 from snakemake.executors import ClusterExecutor, sleep
 from snakemake.common import get_container_image, get_file_hash
+from snakemake.resources import DefaultResources
 
 # https://github.com/googleapis/google-api-python-client/issues/299#issuecomment-343255309
 logging.getLogger("googleapiclient.discovery_cache").setLevel(logging.ERROR)
@@ -72,9 +73,9 @@ class GoogleLifeSciencesExecutor(ClusterExecutor):
 
         exec_job = exec_job or (
             "snakemake {target} --snakefile %s "
-            "--force -j{cores} --keep-target-files --keep-remote "
-            "--latency-wait 0 "
-            "--attempt 1 {use_threads} "
+            "--force --cores {cores} --keep-target-files --keep-remote "
+            "--latency-wait {latency_wait} --scheduler {workflow.scheduler_type} "
+            "--attempt 1 {use_threads} --max-inventory-time 0 "
             "{overwrite_config} {rules} --nocolor "
             "--notemp --no-hooks --nolock " % self.snakefile
         )
@@ -112,10 +113,9 @@ class GoogleLifeSciencesExecutor(ClusterExecutor):
         targz = self._generate_build_source_package()
         self._upload_build_source_package(targz)
 
-        # Save default resources to add later, since we need to add custom
+        # we need to add custom
         # default resources depending on the instance requested
-        self.default_resources = self.workflow.default_resources
-        self.workflow.default_resources.args = None
+        self.default_resources = None
 
         super().__init__(
             workflow,
@@ -273,17 +273,25 @@ class GoogleLifeSciencesExecutor(ClusterExecutor):
         """shutdown deletes build packages if the user didn't request to clean
         up the cache. At this point we've already cancelled running jobs.
         """
-        # Delete build source packages only if user regooglquested no cache
-        if self._save_storage_cache:
-            logger.debug("Requested to save workflow sources, skipping cleanup.")
-        else:
-            for package in self._build_packages:
-                blob = self.bucket.blob(package)
-                if blob.exists():
-                    logger.debug("Deleting blob %s" % package)
-                    blob.delete()
+        from google.api_core import retry
+        from snakemake.remote.GS import google_cloud_retry_predicate
+
+        @retry.Retry(predicate=google_cloud_retry_predicate)
+        def _shutdown():
+            # Delete build source packages only if user regooglquested no cache
+            if self._save_storage_cache:
+                logger.debug("Requested to save workflow sources, skipping cleanup.")
+            else:
+                for package in self._build_packages:
+                    blob = self.bucket.blob(package)
+                    if blob.exists():
+                        logger.debug("Deleting blob %s" % package)
+                        blob.delete()
+
+            # perform additional steps on shutdown if necessary
+
+        _shutdown()
 
-        # perform additional steps on shutdown if necessary
         super().shutdown()
 
     def cancel(self):
@@ -358,8 +366,8 @@ class GoogleLifeSciencesExecutor(ClusterExecutor):
             "found resource request for {} GPUs. This will limit to n1 "
             "instance types.".format(gpu_count)
         )
-        self.workflow.default_resources.parsed["nvidia_gpu"] = gpu_count
-        self.workflow.default_resources.args.append("nvidia_gpu=%s" % gpu_count)
+        self.default_resources.set_resource("nvidia_gpu", gpu_count)
+
         self._machine_type_prefix = self._machine_type_prefix or ""
         if not self._machine_type_prefix.startswith("n1"):
             self._machine_type_prefix = "n1"
@@ -396,10 +404,12 @@ class GoogleLifeSciencesExecutor(ClusterExecutor):
         # Right now, do a best effort mapping of resources to instance types
         cores = job.resources.get("_cores", 1)
         mem_mb = job.resources.get("mem_mb", 15360)
-        disk_mb = job.resources.get("disk_mb", 128000)
 
-        # Convert mb to gb, add buffer of 50
-        disk_gb = math.ceil(disk_mb / 1024) + 10
+        # IOPS performance proportional to disk size
+        disk_mb = job.resources.get("disk_mb", 512000)
+
+        # Convert mb to gb
+        disk_gb = math.ceil(disk_mb / 1024)
 
         # Look for if the user wants an nvidia gpu
         gpu_count = job.resources.get("nvidia_gpu") or job.resources.get("gpu")
@@ -410,13 +420,12 @@ class GoogleLifeSciencesExecutor(ClusterExecutor):
             gpu_count = 1
 
         # Update default resources using decided memory and disk
-        self.workflow.default_resources = self.default_resources
-        self.workflow.default_resources.args = [
-            "mem_mb=%s" % mem_mb,
-            "disk_mb=%s" % disk_mb,
-        ]
-        self.workflow.default_resources.parsed["mem_mb"] = mem_mb
-        self.workflow.default_resources.parsed["disk_mb"] = disk_mb
+
+        self.default_resources = DefaultResources(
+            from_other=self.workflow.default_resources
+        )
+        self.default_resources.set_resource("mem_mb", mem_mb)
+        self.default_resources.set_resource("disk_mb", disk_mb)
 
         # Job resource specification can be overridden by gpu preferences
         self.machine_type_prefix = job.resources.get("machine_type")
@@ -502,11 +511,23 @@ class GoogleLifeSciencesExecutor(ClusterExecutor):
             "Selected machine type {}:{}".format(smallest, selected["description"])
         )
 
+        if job.is_group():
+            preemptible = all(rule in self.preemptible_rules for rule in job.rules)
+            if not preemptible and any(
+                rule in self.preemptible_rules for rule in job.rules
+            ):
+                raise WorkflowError(
+                    "All grouped rules should be homogenously set as preemptible rules"
+                    "(see Defining groups for execution in snakemake documentation)"
+                )
+        else:
+            preemptible = job.rule.name in self.preemptible_rules
+        # We add the size for the image itself (10 GB) to bootDiskSizeGb
         virtual_machine = {
             "machineType": smallest,
             "labels": {"app": "snakemake"},
-            "bootDiskSizeGb": disk_gb,
-            "preemptible": job.rule.name in self.preemptible_rules,
+            "bootDiskSizeGb": disk_gb + 10,
+            "preemptible": preemptible,
         }
 
         # If the user wants gpus, add accelerators here
@@ -575,17 +596,13 @@ class GoogleLifeSciencesExecutor(ClusterExecutor):
         return keepers[smallest]
 
     def _set_snakefile(self):
-        """The snakefile must be a relative path, which cannot be reliably
-        derived from the self.workflow.snakefile as we might have moved
-        execution into a temporary directory, and the initial Snakefile
-        was somewhere else on the system.
+        """The snakefile must be a relative path, which should be derived
+        from the self.workflow.main_snakefile.
         """
-        from snakemake import SNAKEFILE_CHOICES
-
-        for snakefile in SNAKEFILE_CHOICES:
-            if os.path.exists(os.path.join(self.workdir, snakefile)):
-                self.snakefile = snakefile
-                break
+        assert os.path.exists(self.workflow.main_snakefile)
+        self.snakefile = self.workflow.main_snakefile.replace(self.workdir, "").strip(
+            os.sep
+        )
 
     def _set_workflow_sources(self):
         """We only add files from the working directory that are config related
@@ -658,23 +675,30 @@ class GoogleLifeSciencesExecutor(ClusterExecutor):
         """given a .tar.gz created for a workflow, upload it to source/cache
         of Google storage, only if the blob doesn't already exist.
         """
-        # Upload to temporary storage, only if doesn't exist
-        self.pipeline_package = "source/cache/%s" % os.path.basename(targz)
-        blob = self.bucket.blob(self.pipeline_package)
-        logger.debug("build-package=%s" % self.pipeline_package)
-        if not blob.exists():
-            blob.upload_from_filename(targz, content_type="application/gzip")
+        from google.api_core import retry
+        from snakemake.remote.GS import google_cloud_retry_predicate
+
+        @retry.Retry(predicate=google_cloud_retry_predicate)
+        def _upload():
+            # Upload to temporary storage, only if doesn't exist
+            self.pipeline_package = "source/cache/%s" % os.path.basename(targz)
+            blob = self.bucket.blob(self.pipeline_package)
+            logger.debug("build-package=%s" % self.pipeline_package)
+            if not blob.exists():
+                blob.upload_from_filename(targz, content_type="application/gzip")
+
+        _upload()
 
     def _generate_log_action(self, job):
         """generate an action to save the pipeline logs to storage."""
         # script should be changed to this when added to version control!
-        # https://raw.githubusercontent.com/snakemake/snakemake/master/snakemake/executors/google_lifesciences_helper.py
+        # https://raw.githubusercontent.com/snakemake/snakemake/main/snakemake/executors/google_lifesciences_helper.py
 
         # Save logs from /google/logs/output to source/logs in bucket
         commands = [
             "/bin/bash",
             "-c",
-            "wget -O /gls.py https://gist.githubusercontent.com/vsoch/f5a6a6d1894be1e67aa4156c5b40c8e9/raw/a4e9ddbeba20996ca62745fcd4d9ecd7bfa3b311/gls.py && chmod +x /gls.py && source activate snakemake || true && python /gls.py save %s /google/logs %s/%s"
+            "wget -O /gls.py https://raw.githubusercontent.com/snakemake/snakemake/main/snakemake/executors/google_lifesciences_helper.py && chmod +x /gls.py && source activate snakemake || true && python /gls.py save %s /google/logs %s/%s"
             % (self.bucket.name, self.gs_logs, job.name),
         ]
 
@@ -698,17 +722,17 @@ class GoogleLifeSciencesExecutor(ClusterExecutor):
         )
 
         # Now that we've parsed the job resource requirements, add to exec
-        exec_job += self.get_default_resources_args()
+        exec_job += self.get_default_resources_args(self.default_resources)
 
         # script should be changed to this when added to version control!
-        # https://raw.githubusercontent.com/snakemake/snakemake/master/snakemake/executors/google_lifesciences_helper.py
+        # https://raw.githubusercontent.com/snakemake/snakemake/main/snakemake/executors/google_lifesciences_helper.py
         # The full command to download the archive, extract, and run
         # For snakemake bases, we must activate the conda environment, but
         # for custom images we must allow this to fail (hence || true)
         commands = [
             "/bin/bash",
             "-c",
-            "mkdir -p /workdir && cd /workdir && wget -O /download.py https://gist.githubusercontent.com/vsoch/f5a6a6d1894be1e67aa4156c5b40c8e9/raw/a4e9ddbeba20996ca62745fcd4d9ecd7bfa3b311/gls.py && chmod +x /download.py && source activate snakemake || true && python /download.py download %s %s /tmp/workdir.tar.gz && tar -xzvf /tmp/workdir.tar.gz && %s"
+            "mkdir -p /workdir && cd /workdir && wget -O /download.py https://raw.githubusercontent.com/snakemake/snakemake/main/snakemake/executors/google_lifesciences_helper.py && chmod +x /download.py && source activate snakemake || true && python /download.py download %s %s /tmp/workdir.tar.gz && tar -xzvf /tmp/workdir.tar.gz && %s"
             % (self.bucket.name, self.pipeline_package, exec_job),
         ]
 
@@ -793,9 +817,10 @@ class GoogleLifeSciencesExecutor(ClusterExecutor):
         # capabilities - this won't currently work (Singularity in Docker)
         # We either need to add CAPS or run in privileged mode (ehh)
         if job.needs_singularity and self.workflow.use_singularity:
-            logger.warning(
+            raise WorkflowError(
                 "Singularity requires additional capabilities that "
-                "aren't yet supported for standard Docker runs."
+                "aren't yet supported for standard Docker runs, and "
+                "is not supported for the Google Life Sciences executor."
             )
 
         # location looks like: "projects/<project>/locations/<location>"
@@ -880,9 +905,15 @@ class GoogleLifeSciencesExecutor(ClusterExecutor):
                 return self._retry_request(request, timeout * 2, attempts - 1)
             raise ex
         except googleapiclient.errors.HttpError as ex:
+            if attempts > 0:
+                time.sleep(timeout)
+                return self._retry_request(request, timeout * 2, attempts - 1)
             log_verbose_traceback(ex)
             raise ex
         except Exception as ex:
+            if attempts > 0:
+                time.sleep(timeout)
+                return self._retry_request(request, timeout * 2, attempts - 1)
             log_verbose_traceback(ex)
             raise ex
 
