@@ -8,10 +8,10 @@ import re
 import ftplib
 import collections
 from itertools import chain
-from contextlib import contextmanager
 
 # module-specific
-from snakemake.remote import AbstractRemoteProvider, DomainObject
+from snakemake.remote import AbstractRemoteProvider, PooledDomainObject
+
 from snakemake.exceptions import FTPFileException, WorkflowError
 from snakemake.utils import os_sync
 
@@ -32,13 +32,7 @@ class RemoteProvider(AbstractRemoteProvider):
     allows_directories = True
 
     def __init__(
-        self,
-        *args,
-        keep_local=False,
-        stay_on_remote=False,
-        is_default=False,
-        immediate_close=False,
-        **kwargs
+        self, *args, keep_local=False, stay_on_remote=False, is_default=False, **kwargs
     ):
         super(RemoteProvider, self).__init__(
             *args,
@@ -48,8 +42,6 @@ class RemoteProvider(AbstractRemoteProvider):
             **kwargs
         )
 
-        self.immediate_close = immediate_close
-
     @property
     def default_protocol(self):
         """The protocol that is prepended to the path when no protocol is specified."""
@@ -60,9 +52,7 @@ class RemoteProvider(AbstractRemoteProvider):
         """List of valid protocols for this remote provider."""
         return ["ftp://", "ftps://"]
 
-    def remote(
-        self, value, *args, encrypt_data_channel=None, immediate_close=None, **kwargs
-    ):
+    def remote(self, value, *args, encrypt_data_channel=None, **kwargs):
         if isinstance(value, str):
             values = [value]
         elif isinstance(value, collections.abc.Iterable):
@@ -76,13 +66,13 @@ class RemoteProvider(AbstractRemoteProvider):
             match = re.match("^(ftps?)://.+", file)
             if match:
                 (protocol,) = match.groups()
-                if protocol == "ftps" and encrypt_data_channel:
+                if protocol == "ftps" and encrypt_data_channel in [None, False]:
                     raise SyntaxError(
                         "encrypt_data_channel=False cannot be used with a ftps:// url"
                     )
                 if protocol == "ftp" and encrypt_data_channel not in [None, False]:
                     raise SyntaxError(
-                        "encrypt_data_channel=Trie cannot be used with a ftp:// url"
+                        "encrypt_data_channel=True cannot be used with a ftp:// url"
                     )
             else:
                 if encrypt_data_channel:
@@ -90,14 +80,9 @@ class RemoteProvider(AbstractRemoteProvider):
                 else:
                     values[i] = "ftp://" + file
 
-        should_close = immediate_close if immediate_close else self.immediate_close
         values = [
             super(RemoteProvider, self).remote(
-                value,
-                *args,
-                encrypt_data_channel=encrypt_data_channel,
-                immediate_close=should_close,
-                **kwargs
+                value, *args, encrypt_data_channel=encrypt_data_channel, **kwargs
             )
             for value in values
         ]
@@ -107,7 +92,7 @@ class RemoteProvider(AbstractRemoteProvider):
             return values
 
 
-class RemoteObject(DomainObject):
+class RemoteObject(PooledDomainObject):
     """This is a class to interact with an FTP server."""
 
     def __init__(
@@ -116,92 +101,55 @@ class RemoteObject(DomainObject):
         keep_local=False,
         provider=None,
         encrypt_data_channel=False,
-        immediate_close=False,
         **kwargs
     ):
+        # just save to kwargs, but leave in function def so it's explicit
+        kwargs["encrypt_data_channel"] = encrypt_data_channel
         super(RemoteObject, self).__init__(
             *args, keep_local=keep_local, provider=provider, **kwargs
         )
 
-        self.encrypt_data_channel = encrypt_data_channel
-        self.immediate_close = immediate_close
-
-    def close(self):
-        if (
-            hasattr(self, "conn")
-            and isinstance(self.conn, ftputil.FTPHost)
-            and not self.immediate_close
-        ):
-            try:
-                self.conn.keep_alive()
-                self.conn.close()
-            except:
-                pass
-
     # === Implementations of abstract class members ===
 
-    @contextmanager  # makes this a context manager. after 'yield' is __exit__()
-    def ftpc(self):
-        if (
-            not hasattr(self, "conn")
-            or (hasattr(self, "conn") and not isinstance(self.conn, ftputil.FTPHost))
-        ) or self.immediate_close:
-            # if args have been provided to remote(), use them over those given to RemoteProvider()
-            args_to_use = self.provider.args
-            if len(self.args):
-                args_to_use = self.args
+    def get_default_kwargs(self, **defaults):
+        """define defaults beyond thos set in PooledDomainObject"""
+        return super().get_default_kwargs(
+            **{"port": 21, "password": None, "username": None}
+        )
 
-            # use kwargs passed in to remote() to override those given to the RemoteProvider()
-            # default to the host and port given as part of the file, falling back to one specified
-            # as a kwarg to remote() or the RemoteProvider (overriding the latter with the former if both)
-            kwargs_to_use = {}
-            kwargs_to_use["host"] = self.host
-            kwargs_to_use["username"] = None
-            kwargs_to_use["password"] = None
-            kwargs_to_use["port"] = int(self.port) if self.port else 21
-            kwargs_to_use["encrypt_data_channel"] = self.encrypt_data_channel
+    @property
+    def conn_keywords(self):
+        """returns list of keywords relevant to a unique connection"""
+        return ["host", "port", "username", "encrypt_data_channel"]
 
-            for k, v in self.provider.kwargs.items():
-                kwargs_to_use[k] = v
-            for k, v in self.kwargs.items():
-                kwargs_to_use[k] = v
+    def create_connection(self, *args_to_use, **kwargs_to_use):
+        """create a connection to the FTP server using the saved arguments"""
+        ftp_base_class = (
+            ftplib.FTP_TLS if kwargs_to_use["encrypt_data_channel"] else ftplib.FTP
+        )
 
-            ftp_base_class = (
-                ftplib.FTP_TLS if kwargs_to_use["encrypt_data_channel"] else ftplib.FTP
-            )
+        ftp_session_factory = ftputil.session.session_factory(
+            base_class=ftp_base_class,
+            port=kwargs_to_use["port"],
+            encrypt_data_channel=kwargs_to_use["encrypt_data_channel"],
+            debug_level=None,
+        )
 
-            ftp_session_factory = ftputil.session.session_factory(
-                base_class=ftp_base_class,
-                port=kwargs_to_use["port"],
-                encrypt_data_channel=kwargs_to_use["encrypt_data_channel"],
-                debug_level=None,
-            )
+        return ftputil.FTPHost(
+            kwargs_to_use["host"],
+            kwargs_to_use["username"],
+            kwargs_to_use["password"],
+            session_factory=ftp_session_factory,
+        )
 
-            conn = ftputil.FTPHost(
-                kwargs_to_use["host"],
-                kwargs_to_use["username"],
-                kwargs_to_use["password"],
-                session_factory=ftp_session_factory,
-            )
-            if self.immediate_close:
-                yield conn
-            else:
-                self.conn = conn
-                yield self.conn
-        elif not self.immediate_close:
-            yield self.conn
-
-        # after returning from the context manager, close the connection if the scope is local
-        if self.immediate_close:
-            try:
-                conn.keep_alive()
-                conn.close()
-            except:
-                pass
+    def close_connection(self, connection):
+        """close an FTP connection"""
+        connection.keep_alive()
+        connection.close()
 
     def exists(self):
         if self._matched_address:
-            with self.ftpc() as ftpc:
+            with self.connection_pool.item() as ftpc:
                 return ftpc.path.exists(self.remote_path)
             return False
         else:
@@ -212,7 +160,7 @@ class RemoteObject(DomainObject):
 
     def mtime(self):
         if self.exists():
-            with self.ftpc() as ftpc:
+            with self.connection_pool.item() as ftpc:
                 try:
                     # requires write access
                     ftpc.synchronize_times()
@@ -226,13 +174,13 @@ class RemoteObject(DomainObject):
 
     def size(self):
         if self.exists():
-            with self.ftpc() as ftpc:
+            with self.connection_pool.item() as ftpc:
                 return ftpc.path.getsize(self.remote_path)
         else:
             return self._iofile.size_local
 
     def download(self, make_dest_dirs=True):
-        with self.ftpc() as ftpc:
+        with self.connection_pool.item() as ftpc:
             if self.exists():
                 # if the destination path does not exist
                 if make_dest_dirs:
@@ -250,7 +198,7 @@ class RemoteObject(DomainObject):
                 )
 
     def upload(self):
-        with self.ftpc() as ftpc:
+        with self.connection_pool.item() as ftpc:
             ftpc.synchronize_times()
             ftpc.upload(source=self.local_path, target=self.remote_path)
 
@@ -261,7 +209,7 @@ class RemoteObject(DomainObject):
         first_wildcard = self._iofile.constant_prefix()
         dirname = first_wildcard.replace(self.path_prefix, "")
 
-        with self.ftpc() as ftpc:
+        with self.connection_pool.item() as ftpc:
             file_list = [
                 (os.path.join(dirpath, f) if dirpath != "." else f)
                 for dirpath, dirnames, filenames in ftpc.walk(dirname)
