@@ -1,6 +1,6 @@
 __author__ = "Johannes Köster"
-__copyright__ = "Copyright 2015-2019, Johannes Köster"
-__email__ = "koester@jimmy.harvard.edu"
+__copyright__ = "Copyright 2021, Johannes Köster"
+__email__ = "johannes.koester@uni-due.de"
 __license__ = "MIT"
 
 import os
@@ -21,7 +21,7 @@ from collections import namedtuple, defaultdict
 from itertools import accumulate, chain
 import urllib.parse
 import hashlib
-from zipfile import ZipFile
+from zipfile import ZipFile, ZIP_DEFLATED
 from pathlib import Path
 
 import requests
@@ -30,7 +30,7 @@ from docutils.parsers.rst.directives.images import Image, Figure
 from docutils.parsers.rst import directives
 from docutils.core import publish_file, publish_parts
 
-from snakemake import script, wrapper
+from snakemake import script, wrapper, notebook
 from snakemake.utils import format
 from snakemake.logging import logger
 from snakemake.io import (
@@ -39,6 +39,7 @@ from snakemake.io import (
     glob_wildcards,
     Wildcards,
     apply_wildcards,
+    contains_wildcard,
 )
 from snakemake.exceptions import WorkflowError
 from snakemake.script import Snakemake
@@ -242,6 +243,7 @@ class RuleRecord:
         self.n_jobs = 1
         self.id = uuid.uuid4()
 
+    @lazy_property
     def code(self):
         try:
             from pygments.lexers import get_lexer_by_name
@@ -252,34 +254,65 @@ class RuleRecord:
             raise WorkflowError(
                 "Python package pygments must be installed to create reports."
             )
-        source, language = None, None
+        sources, language = None, None
         if self._rule.shellcmd is not None:
-            source = self._rule.shellcmd
+            sources = [self._rule.shellcmd]
             language = "bash"
-        elif self._rule.script is not None:
+        elif self._rule.script is not None and not contains_wildcard(self._rule.script):
             logger.info("Loading script code for rule {}".format(self.name))
-            _, source, language = script.get_source(
-                self._rule.script, self._rule.basedir
+            _, source, language, _ = script.get_source(
+                self._rule.script, self._rule.workflow.sourcecache, self._rule.basedir
             )
-            source = source.decode()
-        elif self._rule.wrapper is not None:
+            sources = [source.decode()]
+        elif self._rule.wrapper is not None and not contains_wildcard(
+            self._rule.wrapper
+        ):
             logger.info("Loading wrapper code for rule {}".format(self.name))
-            _, source, language = script.get_source(
+            _, source, language, _ = script.get_source(
                 wrapper.get_script(
-                    self._rule.wrapper, prefix=self._rule.workflow.wrapper_prefix
-                )
+                    self._rule.wrapper,
+                    self._rule.workflow.sourcecache,
+                    prefix=self._rule.workflow.wrapper_prefix,
+                ),
+                self._rule.workflow.sourcecache,
             )
-            source = source.decode()
+            sources = [source.decode()]
+        elif self._rule.notebook is not None and not contains_wildcard(
+            self._rule.notebook
+        ):
+            _, source, language, _ = script.get_source(
+                self._rule.notebook,
+                self._rule.workflow.sourcecache,
+                self._rule.basedir,
+                wildcards=self.wildcards,
+                params=self.params,
+            )
+            language = language.split("_")[1]
+            sources = notebook.get_cell_sources(source)
+        else:
+            # A run directive. There is no easy way yet to obtain
+            # the actual uncompiled source code.
+            sources = []
+            language = "python"
 
         try:
             lexer = get_lexer_by_name(language)
-            return highlight(
-                source,
-                lexer,
-                HtmlFormatter(linenos=True, cssclass="source", wrapcode=True),
-            )
+
+            highlighted = [
+                highlight(
+                    source,
+                    lexer,
+                    HtmlFormatter(linenos=True, cssclass="source", wrapcode=True),
+                )
+                for source in sources
+            ]
+
+            return highlighted
         except pygments.util.ClassNotFound:
-            return "<pre><code>source</code></pre>"
+            return [
+                '<pre class="source"><code>{}</code></pre>'.format(source)
+                for source in sources
+            ]
 
     def add(self, job_rec):
         self.n_jobs += 1
@@ -302,7 +335,7 @@ class RuleRecord:
 
 class ConfigfileRecord:
     def __init__(self, configfile):
-        self.name = configfile
+        self.path = Path(configfile)
 
     def code(self):
         try:
@@ -314,19 +347,25 @@ class ConfigfileRecord:
                 "Python package pygments must be installed to create reports."
             )
 
-        language = (
-            "yaml"
-            if self.name.endswith(".yaml") or self.name.endswith(".yml")
-            else "json"
-        )
-        lexer = get_lexer_by_name(language)
-        with open(self.name) as f:
-            return highlight(
-                f.read(),
-                lexer,
-                HtmlFormatter(linenos=True, cssclass="source", wrapcode=True),
+        file_ext = self.path.suffix
+        if file_ext in (".yml", ".yaml"):
+            language = "yaml"
+        elif file_ext == ".json":
+            language = "json"
+        else:
+            raise ValueError(
+                "Config file extension {} is not supported - must be YAML or JSON".format(
+                    file_ext
+                )
             )
 
+        lexer = get_lexer_by_name(language)
+        return highlight(
+            self.path.read_text(),
+            lexer,
+            HtmlFormatter(linenos=True, cssclass="source", wrapcode=True),
+        )
+
 
 class JobRecord:
     def __init__(self):
@@ -347,9 +386,13 @@ class FileRecord:
         caption,
         env,
         category,
+        workflow,
         wildcards_overwrite=None,
         mode_embedded=True,
+        aux_files=None,
+        name_overwrite=None,
     ):
+        self.name_overwrite = name_overwrite
         self.mode_embedded = mode_embedded
         self.path = path
         self.target = os.path.basename(path)
@@ -357,6 +400,7 @@ class FileRecord:
         logger.info("Adding {} ({:.2g} MB).".format(self.name, self.size / 1e6))
         self.raw_caption = caption
         self.mime, _ = mime_from_file(self.path)
+        self.workflow = workflow
 
         h = hashlib.sha256()
         h.update(path.encode())
@@ -372,43 +416,7 @@ class FileRecord:
         )
         self.category = category
 
-        self.table_content = None
-        if self.is_table:
-            if self.size > 1e6:
-                logger.warning(
-                    "Table {} >1MB. Rendering as generic file.".format(self.path)
-                )
-            else:
-                with open(self.path) as table:
-                    dialect = None
-                    for prefix in range(10, 17):
-                        try:
-                            table.seek(0)
-                            dialect = csv.Sniffer().sniff(table.read(prefix))
-                            break
-                        except csv.Error:
-                            pass
-                        except UnicodeDecodeError:
-                            # table is not readable as UTF-8
-                            break
-                    if dialect is None:
-                        logger.warning(
-                            "Failed to infer CSV/TSV dialect from table {}. "
-                            "Rendering as generic file.".format(self.path)
-                        )
-                    else:
-                        table.seek(0)
-                        reader = csv.reader(table, dialect)
-                        columns = next(reader)
-                        table = map(lambda row: list(map(num_if_possible, row)), reader)
-                        template = env.get_template("table.html")
-                        html = template.render(
-                            columns=columns, table=table, name=self.name
-                        ).encode()
-
-                        self.table_content = html
-                        self.mime = "text/html"
-                        self.path = os.path.basename(self.path) + ".html"
+        self.aux_files = aux_files or []
 
         self.data_uri = self._data_uri()
         self.png_uri = self._png_uri()
@@ -417,7 +425,7 @@ class FileRecord:
     def png_content(self):
         assert self.is_img
 
-        convert = shutil.which("convert")
+        convert = shutil.which("magick")
         if convert is not None:
             try:
                 # 2048 aims at a reasonable balance between what displays
@@ -428,7 +436,8 @@ class FileRecord:
                 # '>' means only larger images scaled down to within max-dimensions
                 max_spec = max_width + "x" + max_height + ">"
                 png = sp.check_output(
-                    ["convert", "-resize", max_spec, self.path, "png:-"], stderr=sp.PIPE
+                    ["magick", "convert", "-resize", max_spec, self.path, "png:-"],
+                    stderr=sp.PIPE,
                 )
                 return png
             except sp.CalledProcessError as e:
@@ -458,12 +467,9 @@ class FileRecord:
 
     def _data_uri(self):
         if self.mode_embedded:
-            if self.table_content is not None:
-                return data_uri(self.table_content, self.path, self.mime)
-            else:
-                return data_uri_from_file(self.path)
+            return data_uri_from_file(self.path)
         else:
-            return os.path.join("data/raw", self.id, self.name)
+            return os.path.join("data/raw", self.id, self.filename)
 
     def render(self, env, rst_links, categories, files):
         if self.raw_caption is not None:
@@ -489,7 +495,9 @@ class FileRecord:
             )
 
             try:
-                caption = open(self.raw_caption).read() + rst_links
+                caption = (
+                    self.workflow.sourcecache.open(self.raw_caption).read() + rst_links
+                )
                 caption = env.from_string(caption).render(
                     snakemake=snakemake, categories=categories, files=files
                 )
@@ -539,6 +547,12 @@ class FileRecord:
 
     @property
     def name(self):
+        if self.name_overwrite:
+            return self.name_overwrite
+        return os.path.basename(self.path)
+
+    @property
+    def filename(self):
         return os.path.basename(self.path)
 
 
@@ -630,7 +644,9 @@ def auto_report(dag, path, stylesheet=None):
                     )
                 report_obj = get_flag_value(f, "report")
 
-                def register_file(f, wildcards_overwrite=None):
+                def register_file(
+                    f, wildcards_overwrite=None, aux_files=None, name_overwrite=None
+                ):
                     wildcards = wildcards_overwrite or job.wildcards
                     category = Category(
                         report_obj.category, wildcards=wildcards, job=job
@@ -646,36 +662,72 @@ def auto_report(dag, path, stylesheet=None):
                             report_obj.caption,
                             env,
                             category,
+                            dag.workflow,
                             wildcards_overwrite=wildcards_overwrite,
                             mode_embedded=mode_embedded,
+                            aux_files=aux_files,
+                            name_overwrite=name_overwrite,
                         )
                     )
                     recorded_files.add(f)
 
                 if os.path.isfile(f):
                     register_file(f)
-                if os.path.isdir(f):
-                    if not isinstance(report_obj.patterns, list):
-                        raise WorkflowError(
-                            "Invalid patterns given for report. Must be list.",
-                            rule=job.rule,
+                elif os.path.isdir(f):
+                    if report_obj.htmlindex:
+                        if mode_embedded:
+                            raise WorkflowError(
+                                "Directory marked for report specifies htmlindex. "
+                                "This is unsupported when requesting a pure HTML report. "
+                                "Please use store as zip instead (--report report.zip)."
+                            )
+                        aux_files = []
+                        index_found = False
+                        for root, dirs, files in os.walk(f):
+                            for name in files:
+                                if name != ".snakemake_timestamp":
+                                    filepath = os.path.join(root, name)
+                                    if (
+                                        os.path.relpath(filepath, f)
+                                        != report_obj.htmlindex
+                                    ):
+                                        aux_files.append(filepath)
+                                    else:
+                                        index_found = True
+                        if not index_found:
+                            raise WorkflowError(
+                                "Given htmlindex {} not found in directory "
+                                "marked for report".format(report_obj.htmlindex)
+                            )
+                        register_file(
+                            os.path.join(f, report_obj.htmlindex),
+                            aux_files=aux_files,
+                            name_overwrite="{}.html".format(os.path.basename(f)),
                         )
-                    if not report_obj.patterns:
+                    elif report_obj.patterns:
+                        if not isinstance(report_obj.patterns, list):
+                            raise WorkflowError(
+                                "Invalid patterns given for report. Must be list.",
+                                rule=job.rule,
+                            )
+
+                        for pattern in report_obj.patterns:
+                            pattern = os.path.join(f, pattern)
+                            wildcards = glob_wildcards(pattern)._asdict()
+                            names = wildcards.keys()
+                            for w in zip(*wildcards.values()):
+                                w = dict(zip(names, w))
+                                w.update(job.wildcards_dict)
+                                w = Wildcards(fromdict=w)
+                                f = apply_wildcards(pattern, w)
+                                register_file(f, wildcards_overwrite=w)
+                    else:
                         raise WorkflowError(
-                            "Directory marked for report but no file patterns given via patterns=[...]. "
+                            "Directory marked for report but neither file patterns "
+                            "given via patterns=[...], nor htmlindex given. "
                             "See report documentation.",
                             rule=job.rule,
                         )
-                    for pattern in report_obj.patterns:
-                        pattern = os.path.join(f, pattern)
-                        wildcards = glob_wildcards(pattern)._asdict()
-                        names = wildcards.keys()
-                        for w in zip(*wildcards.values()):
-                            w = dict(zip(names, w))
-                            w.update(job.wildcards_dict)
-                            w = Wildcards(fromdict=w)
-                            f = apply_wildcards(pattern, w)
-                            register_file(f, wildcards_overwrite=w)
 
         for f in job.expanded_output:
             meta = persistence.metadata(f)
@@ -687,14 +739,20 @@ def auto_report(dag, path, stylesheet=None):
                     "warning.".format(f)
                 )
                 continue
+
+            def get_time(rectime, metatime, sel_func):
+                if metatime is None:
+                    return rectime
+                return sel_func(metatime, rectime)
+
             try:
                 job_hash = meta["job_hash"]
                 rule = meta["rule"]
                 rec = records[(job_hash, rule)]
                 rec.rule = rule
                 rec.job = job
-                rec.starttime = min(rec.starttime, meta["starttime"])
-                rec.endtime = max(rec.endtime, meta["endtime"])
+                rec.starttime = get_time(rec.starttime, meta["starttime"], min)
+                rec.endtime = get_time(rec.endtime, meta["endtime"], max)
                 rec.conda_env_file = None
                 rec.conda_env = meta["conda_env"]
                 rec.container_img_url = meta["container_img_url"]
@@ -716,12 +774,18 @@ def auto_report(dag, path, stylesheet=None):
         for rec in sorted(records.values(), key=lambda rec: rec.rule)
     ]
 
+    def get_datetime(rectime):
+        try:
+            return datetime.datetime.fromtimestamp(rectime).isoformat()
+        except OSError:
+            return None
+
     # prepare end times
     timeline = [
         {
             "rule": rec.rule,
-            "starttime": datetime.datetime.fromtimestamp(rec.starttime).isoformat(),
-            "endtime": datetime.datetime.fromtimestamp(rec.endtime).isoformat(),
+            "starttime": get_datetime(rec.starttime),
+            "endtime": get_datetime(rec.endtime),
         }
         for rec in sorted(records.values(), key=lambda rec: rec.rule)
     ]
@@ -778,7 +842,7 @@ def auto_report(dag, path, stylesheet=None):
     # global description
     text = ""
     if dag.workflow.report_text:
-        with open(dag.workflow.report_text) as f:
+        with dag.workflow.sourcecache.open(dag.workflow.report_text) as f:
 
             class Snakemake:
                 config = dag.workflow.config
@@ -807,7 +871,7 @@ def auto_report(dag, path, stylesheet=None):
             "Python package pygments must be installed to create reports."
         )
 
-    template = env.get_template("report.html")
+    template = env.get_template("report.html.jinja2")
 
     logger.info("Downloading resources and rendering HTML.")
 
@@ -833,27 +897,33 @@ def auto_report(dag, path, stylesheet=None):
     # TODO look into supporting .WARC format, also see (https://webrecorder.io)
 
     if not mode_embedded:
-        with ZipFile(path, mode="w") as zipout:
+        with ZipFile(path, compression=ZIP_DEFLATED, mode="w") as zipout:
             folder = Path(Path(path).stem)
             # store results in data folder
             for subcats in results.values():
                 for catresults in subcats.values():
                     for result in catresults:
                         # write raw data
-                        if result.table_content is not None:
-                            zipout.writestr(
-                                str(folder.joinpath(result.data_uri)),
-                                result.table_content,
-                            )
-                        else:
-                            zipout.write(
-                                result.path, str(folder.joinpath(result.data_uri))
-                            )
+                        zipout.write(result.path, str(folder.joinpath(result.data_uri)))
                         # write thumbnail
                         if result.is_img and result.png_content:
                             zipout.writestr(
                                 str(folder.joinpath(result.png_uri)), result.png_content
                             )
+                        # write aux files
+                        parent = folder.joinpath(result.data_uri).parent
+                        for aux_path in result.aux_files:
+                            # print(aux_path, parent, str(parent.joinpath(os.path.relpath(aux_path, os.path.dirname(result.path)))))
+                            zipout.write(
+                                aux_path,
+                                str(
+                                    parent.joinpath(
+                                        os.path.relpath(
+                                            aux_path, os.path.dirname(result.path)
+                                        )
+                                    )
+                                ),
+                            )
 
             # write report html
             zipout.writestr(str(folder.joinpath("report.html")), rendered)
