@@ -1,10 +1,11 @@
 __author__ = "Johannes Köster"
-__copyright__ = "Copyright 2015-2019, Johannes Köster"
-__email__ = "koester@jimmy.harvard.edu"
+__copyright__ = "Copyright 2021, Johannes Köster"
+__email__ = "johannes.koester@uni-due.de"
 __license__ = "MIT"
 
 import os
 import re
+from snakemake.path_modifier import PATH_MODIFIER_FLAG
 import sys
 import inspect
 import sre_constants
@@ -12,6 +13,7 @@ import collections
 from urllib.parse import urljoin
 from pathlib import Path
 from itertools import chain
+from functools import partial
 
 from snakemake.io import (
     IOFile,
@@ -23,6 +25,8 @@ from snakemake.io import (
     AnnotatedString,
     contains_wildcard_constraints,
     update_wildcard_constraints,
+    flag,
+    get_flag_value,
 )
 from snakemake.io import (
     expand,
@@ -51,7 +55,8 @@ from snakemake.exceptions import (
     IncompleteCheckpointException,
 )
 from snakemake.logging import logger
-from snakemake.common import Mode, lazy_property, TBDInt
+from snakemake.common import Mode, ON_WINDOWS, lazy_property, TBDString
+import snakemake.io
 
 
 class Rule:
@@ -87,6 +92,7 @@ class Rule:
             self._benchmark = None
             self._conda_env = None
             self._container_img = None
+            self.is_containerized = False
             self.env_modules = None
             self.group = None
             self._wildcard_names = None
@@ -99,10 +105,13 @@ class Rule:
             self.wrapper = None
             self.cwl = None
             self.norun = False
+            self.is_handover = False
             self.is_branched = False
             self.is_checkpoint = False
             self.restart_times = 0
             self.basedir = None
+            self.path_modifer = None
+            self.ruleinfo = None
         elif len(args) == 1:
             other = args[0]
             self.name = other.name
@@ -128,6 +137,7 @@ class Rule:
             self._benchmark = other._benchmark
             self._conda_env = other._conda_env
             self._container_img = other._container_img
+            self.is_containerized = other.is_containerized
             self.env_modules = other.env_modules
             self.group = other.group
             self._wildcard_names = (
@@ -144,10 +154,13 @@ class Rule:
             self.wrapper = other.wrapper
             self.cwl = other.cwl
             self.norun = other.norun
+            self.is_handover = other.is_handover
             self.is_branched = True
             self.is_checkpoint = other.is_checkpoint
             self.restart_times = other.restart_times
             self.basedir = other.basedir
+            self.path_modifier = other.path_modifier
+            self.ruleinfo = other.ruleinfo
 
     def dynamic_branch(self, wildcards, input=True):
         def get_io(rule):
@@ -317,7 +330,7 @@ class Rule:
         if isinstance(benchmark, Path):
             benchmark = str(benchmark)
         if not callable(benchmark):
-            benchmark = self.apply_default_remote(benchmark)
+            benchmark = self.apply_path_modifier(benchmark, property="benchmark")
             benchmark = self._update_item_wildcard_constraints(benchmark)
 
         self._benchmark = IOFile(benchmark, rule=self)
@@ -329,7 +342,16 @@ class Rule:
 
     @conda_env.setter
     def conda_env(self, conda_env):
-        self._conda_env = IOFile(conda_env, rule=self)
+        from snakemake.deployment.conda import (
+            is_conda_env_file,
+            CondaEnvFileSpec,
+            CondaEnvNameSpec,
+        )
+
+        if is_conda_env_file(conda_env):
+            self._conda_env = CondaEnvFileSpec(conda_env, rule=self)
+        else:
+            self._conda_env = CondaEnvNameSpec(conda_env)
 
     @property
     def container_img(self):
@@ -431,21 +453,9 @@ class Rule:
                 )
             seen[value] = name or idx
 
-    def apply_default_remote(self, item):
-        def is_annotated_callable(value):
-            if isinstance(value, AnnotatedString):
-                return bool(value.callable)
-
-        def apply(value):
-            if (
-                not is_flagged(value, "remote_object")
-                and not is_flagged(value, "local")
-                and not is_annotated_callable(value)
-                and self.workflow.default_remote_provider is not None
-            ):
-                return self.workflow.apply_default_remote(value)
-            else:
-                return value
+    def apply_path_modifier(self, item, property=None):
+        assert self.path_modifier is not None
+        apply = partial(self.path_modifier.modify, property=property)
 
         assert not callable(item)
         if isinstance(item, dict):
@@ -485,11 +495,24 @@ class Rule:
         name     -- an optional name for the item
         """
         inoutput = self.output if output else self.input
+
         # Check to see if the item is a path, if so, just make it a string
         if isinstance(item, Path):
             item = str(item)
         if isinstance(item, str):
-            item = self.apply_default_remote(item)
+            if ON_WINDOWS:
+                if isinstance(item, (_IOFile, AnnotatedString)):
+                    item = item.new_from(item.replace(os.sep, os.altsep))
+                else:
+                    item = item.replace(os.sep, os.altsep)
+
+            rule_dependency = None
+            if isinstance(item, _IOFile) and item.rule and item in item.rule.output:
+                rule_dependency = item.rule
+
+            item = self.apply_path_modifier(
+                item, property="output" if output else "input"
+            )
 
             # Check to see that all flags are valid
             # Note that "remote", "dynamic", and "expand" are valid for both inputs and outputs.
@@ -516,8 +539,8 @@ class Rule:
                         )
 
             # add the rule to the dependencies
-            if isinstance(item, _IOFile) and item.rule and item in item.rule.output:
-                self.dependencies[item] = item.rule
+            if rule_dependency is not None:
+                self.dependencies[item] = rule_dependency
             if output:
                 item = self._update_item_wildcard_constraints(item)
             else:
@@ -525,9 +548,19 @@ class Rule:
                     contains_wildcard_constraints(item)
                     and self.workflow.mode != Mode.subprocess
                 ):
-                    logger.warning("Wildcard constraints in inputs are ignored.")
+                    logger.warning(
+                        "Wildcard constraints in inputs are ignored. (rule: {})".format(
+                            self
+                        )
+                    )
+
+            if self.workflow.all_temp and output:
+                # mark as temp if all output files shall be marked as temp
+                item = snakemake.io.flag(item, "temp")
+
             # record rule if this is an output file output
             _item = IOFile(item, rule=self)
+
             if is_flagged(item, "temp"):
                 if output:
                     self.temp_output.add(_item)
@@ -546,10 +579,11 @@ class Rule:
                 report_obj = item.flags["report"]
                 if report_obj.caption is not None:
                     r = ReportObject(
-                        os.path.join(self.workflow.current_basedir, report_obj.caption),
+                        self.workflow.current_basedir.join(report_obj.caption),
                         report_obj.category,
                         report_obj.subcategory,
                         report_obj.patterns,
+                        report_obj.htmlindex,
                     )
                     item.flags["report"] = r
             if is_flagged(item, "subworkflow"):
@@ -631,7 +665,7 @@ class Rule:
             item = str(item)
         if isinstance(item, str) or callable(item):
             if not callable(item):
-                item = self.apply_default_remote(item)
+                item = self.apply_path_modifier(item, property="log")
                 item = self._update_item_wildcard_constraints(item)
 
             self.log.append(IOFile(item, rule=self) if isinstance(item, str) else item)
@@ -679,6 +713,14 @@ class Rule:
         except IncompleteCheckpointException as e:
             value = incomplete_checkpoint_func(e)
             incomplete = True
+        except FileNotFoundError as e:
+            # Function evaluation can depend on input files. Since expansion can happen during dryrun,
+            # where input files are not yet present, we need to skip such cases and
+            # mark them as <TBD>.
+            if "input" in aux_params and e.filename in aux_params["input"]:
+                value = TBDString()
+            else:
+                raise e
         except (Exception, BaseException) as e:
             if raw_exceptions:
                 raise e
@@ -697,7 +739,8 @@ class Rule:
         mapping=None,
         no_flattening=False,
         aux_params=None,
-        apply_default_remote=True,
+        apply_path_modifier=True,
+        property=None,
         incomplete_checkpoint_func=lambda e: None,
         allow_unpack=True,
     ):
@@ -718,8 +761,8 @@ class Rule:
                     is_unpack=is_unpack,
                     **aux_params
                 )
-                if apply_default_remote:
-                    item = self.apply_default_remote(item)
+                if apply_path_modifier:
+                    item = self.apply_path_modifier(item, property=property)
 
             if is_unpack and not incomplete:
                 if not allow_unpack:
@@ -752,7 +795,11 @@ class Rule:
                     item = [item]
                     is_iterable = False
                 for item_ in item:
-                    if check_return_type and not isinstance(item_, str):
+                    if (
+                        check_return_type
+                        and not isinstance(item_, str)
+                        and not isinstance(item_, Path)
+                    ):
                         raise WorkflowError(
                             "Function did not return str or list " "of str.", rule=self
                         )
@@ -771,7 +818,7 @@ class Rule:
         def concretize_iofile(f, wildcards, is_from_callable):
             if is_from_callable:
                 if isinstance(f, Path):
-                    f = str(Path)
+                    f = str(f)
                 return IOFile(f, rule=self).apply_wildcards(
                     wildcards,
                     fill_missing=f in self.dynamic_input,
@@ -799,6 +846,7 @@ class Rule:
                 concretize=concretize_iofile,
                 mapping=mapping,
                 incomplete_checkpoint_func=handle_incomplete_checkpoint,
+                property="input",
             )
         except WildcardError as e:
             raise WildcardError(
@@ -835,6 +883,18 @@ class Rule:
                     ]
             return p
 
+        def handle_incomplete_checkpoint(exception):
+            """If checkpoint is incomplete, target it such that it is completed
+            before this rule gets executed."""
+            print(exception.targetfile)
+            if exception.targetfile in input:
+                return TBDString()
+            else:
+                raise WorkflowError(
+                    "Rule parameter depends on checkpoint but checkpoint output is not defined as input file for the rule. "
+                    "Please add the output of the respective checkpoint to the rule inputs."
+                )
+
         params = Params()
         try:
             # When applying wildcards to params, the return type need not be
@@ -848,14 +908,15 @@ class Rule:
                 omit_callable=omit_callable,
                 allow_unpack=False,
                 no_flattening=True,
-                apply_default_remote=False,
+                apply_path_modifier=False,
+                property="params",
                 aux_params={
                     "input": input._plainstrings(),
                     "resources": resources,
                     "output": output._plainstrings(),
                     "threads": resources._cores,
                 },
-                incomplete_checkpoint_func=lambda e: "<incomplete checkpoint>",
+                incomplete_checkpoint_func=handle_incomplete_checkpoint,
             )
         except WildcardError as e:
             raise WildcardError(
@@ -896,7 +957,7 @@ class Rule:
 
         try:
             self._apply_wildcards(
-                log, self.log, wildcards, concretize=concretize_logfile
+                log, self.log, wildcards, concretize=concretize_logfile, property="log"
             )
         except WildcardError as e:
             raise WildcardError(
@@ -934,35 +995,31 @@ class Rule:
         def apply(name, res, threads=None):
             if callable(res):
                 aux = dict(rulename=self.name)
-                if threads:
+                if threads is not None:
                     aux["threads"] = threads
                 try:
-                    try:
-                        res, _ = self.apply_input_function(
-                            res,
-                            wildcards,
-                            input=input,
-                            attempt=attempt,
-                            incomplete_checkpoint_func=lambda e: 0,
-                            raw_exceptions=True,
-                            **aux
-                        )
-                    except FileNotFoundError as e:
-                        # Resources can depend on input files. Since expansion can happen during dryrun,
-                        # where input files are not yet present, we need to skip such resources and
-                        # mark them as [TBD].
-                        if e.filename in input:
-                            # use zero for resource if it cannot yet be determined
-                            res = TBDInt(0)
-                        else:
-                            raise e
+                    res, _ = self.apply_input_function(
+                        res,
+                        wildcards,
+                        input=input,
+                        attempt=attempt,
+                        incomplete_checkpoint_func=lambda e: 0,
+                        raw_exceptions=True,
+                        **aux
+                    )
                 except (Exception, BaseException) as e:
                     raise InputFunctionException(e, rule=self, wildcards=wildcards)
 
-                if not isinstance(res, int) and not isinstance(res, str):
-                    raise WorkflowError(
-                        "Resources function did not return int or str.", rule=self
-                    )
+            if isinstance(res, float):
+                # round to integer
+                res = int(round(res))
+
+            if not isinstance(res, int) and not isinstance(res, str):
+                raise WorkflowError(
+                    "Resources function did not return int, float (floats are "
+                    "rouded to the nearest integer), or str.",
+                    rule=self,
+                )
             if isinstance(res, int):
                 global_res = self.workflow.global_resources.get(name, res)
                 if global_res is not None:
@@ -970,6 +1027,8 @@ class Rule:
             return res
 
         threads = apply("_cores", self.resources["_cores"])
+        if self.workflow.max_threads is not None:
+            threads = min(threads, self.workflow.max_threads)
         resources["_cores"] = threads
 
         for name, res in self.resources.items():
@@ -1150,6 +1209,9 @@ class RuleProxy:
             prefix = self.rule.workflow.default_remote_prefix
             # remove constraints and turn this into a plain string
             cleaned = strip_wildcard_constraints(f)
+
+            modified_by = get_flag_value(f, PATH_MODIFIER_FLAG)
+
             if (
                 self.rule.workflow.default_remote_provider is not None
                 and f.startswith(prefix)
@@ -1161,6 +1223,9 @@ class RuleProxy:
                 cleaned = IOFile(AnnotatedString(cleaned), rule=self.rule)
                 cleaned.clone_remote_object(f)
 
+            if modified_by is not None:
+                cleaned.flags[PATH_MODIFIER_FLAG] = modified_by
+
             return cleaned
 
         files = Namedlist(files, custom_map=cleanup)
