@@ -1,6 +1,6 @@
 __author__ = "Johannes Köster"
-__copyright__ = "Copyright 2015-2019, Johannes Köster"
-__email__ = "koester@jimmy.harvard.edu"
+__copyright__ = "Copyright 2021, Johannes Köster"
+__email__ = "johannes.koester@uni-due.de"
 __license__ = "MIT"
 
 import os, signal, sys
@@ -8,6 +8,7 @@ import threading
 import operator
 import time
 import math
+import asyncio
 
 from functools import partial
 from collections import defaultdict
@@ -24,9 +25,10 @@ from snakemake.executors import (
     TibannaExecutor,
 )
 from snakemake.executors.google_lifesciences import GoogleLifeSciencesExecutor
+from snakemake.executors.ga4gh_tes import TaskExecutionServiceExecutor
 from snakemake.exceptions import RuleException, WorkflowError, print_exception
 from snakemake.shell import shell
-
+from snakemake.common import async_run
 from snakemake.logging import logger
 
 from fractions import Fraction
@@ -54,7 +56,6 @@ class JobScheduler:
         self,
         workflow,
         dag,
-        cores,
         local_cores=1,
         dryrun=False,
         touch=False,
@@ -72,6 +73,7 @@ class JobScheduler:
         google_lifesciences_regions=None,
         google_lifesciences_location=None,
         google_lifesciences_cache=False,
+        tes=None,
         precommand="",
         preemption_default=None,
         preemptible_rules=None,
@@ -88,11 +90,15 @@ class JobScheduler:
         force_use_threads=False,
         assume_shared_fs=True,
         keepincomplete=False,
+        keepmetadata=True,
         scheduler_type=None,
+        scheduler_ilp_solver=None,
     ):
-        """ Create a new instance of KnapsackJobScheduler. """
+        """Create a new instance of KnapsackJobScheduler."""
         from ratelimiter import RateLimiter
 
+        cores = workflow.global_resources["_cores"]
+
         self.cluster = cluster
         self.cluster_config = cluster_config
         self.cluster_sync = cluster_sync
@@ -108,12 +114,24 @@ class JobScheduler:
         self.greediness = 1
         self.max_jobs_per_second = max_jobs_per_second
         self.keepincomplete = keepincomplete
+        self.keepmetadata = keepmetadata
         self.scheduler_type = scheduler_type
+        self.scheduler_ilp_solver = scheduler_ilp_solver
+        self._tofinish = []
+        self._toerror = []
+        self.handle_job_success = True
+        self.update_resources = True
+        self.print_progress = not self.quiet and not self.dryrun
+        self.update_dynamic = not self.dryrun
 
         self.global_resources = {
             name: (sys.maxsize if res is None else res)
             for name, res in workflow.global_resources.items()
         }
+
+        if workflow.global_resources["_nodes"] is not None:
+            # Do not restrict cores locally if nodes are used (i.e. in case of cluster/cloud submission).
+            self.global_resources["_cores"] = sys.maxsize
         self.resources = dict(self.global_resources)
 
         use_threads = (
@@ -127,14 +145,12 @@ class JobScheduler:
         self._lock = threading.Lock()
 
         self._errors = False
+        self._executor_error = None
         self._finished = False
         self._job_queue = None
+        self._last_job_selection_empty = False
         self._submit_callback = self._noop
-        self._finish_callback = partial(
-            self._proceed,
-            update_dynamic=not self.dryrun,
-            print_progress=not self.quiet and not self.dryrun,
-        )
+        self._finish_callback = self._proceed
 
         self._local_executor = None
         if dryrun:
@@ -169,6 +185,7 @@ class JobScheduler:
                     latency_wait=latency_wait,
                     cores=local_cores,
                     keepincomplete=keepincomplete,
+                    keepmetadata=keepmetadata,
                 )
             if cluster or cluster_sync:
                 if cluster_sync:
@@ -193,15 +210,13 @@ class JobScheduler:
                     latency_wait=latency_wait,
                     assume_shared_fs=assume_shared_fs,
                     keepincomplete=keepincomplete,
+                    keepmetadata=keepmetadata,
                 )
                 if workflow.immediate_submit:
-                    self._submit_callback = partial(
-                        self._proceed,
-                        update_dynamic=False,
-                        print_progress=False,
-                        update_resources=False,
-                        handle_job_success=False,
-                    )
+                    self.update_dynamic = False
+                    self.print_progress = False
+                    self.update_resources = False
+                    self.handle_job_success = False
             else:
                 self._executor = DRMAAExecutor(
                     workflow,
@@ -218,6 +233,7 @@ class JobScheduler:
                     assume_shared_fs=assume_shared_fs,
                     max_status_checks_per_second=max_status_checks_per_second,
                     keepincomplete=keepincomplete,
+                    keepmetadata=keepmetadata,
                 )
         elif kubernetes:
             self._local_executor = CPUExecutor(
@@ -230,6 +246,7 @@ class JobScheduler:
                 latency_wait=latency_wait,
                 cores=local_cores,
                 keepincomplete=keepincomplete,
+                keepmetadata=keepmetadata,
             )
 
             self._executor = KubernetesExecutor(
@@ -243,6 +260,7 @@ class JobScheduler:
                 latency_wait=latency_wait,
                 cluster_config=cluster_config,
                 keepincomplete=keepincomplete,
+                keepmetadata=keepmetadata,
             )
         elif tibanna:
             self._local_executor = CPUExecutor(
@@ -256,6 +274,7 @@ class JobScheduler:
                 latency_wait=latency_wait,
                 cores=local_cores,
                 keepincomplete=keepincomplete,
+                keepmetadata=keepmetadata,
             )
 
             self._executor = TibannaExecutor(
@@ -271,6 +290,7 @@ class JobScheduler:
                 printshellcmds=printshellcmds,
                 latency_wait=latency_wait,
                 keepincomplete=keepincomplete,
+                keepmetadata=keepmetadata,
             )
         elif google_lifesciences:
             self._local_executor = CPUExecutor(
@@ -299,6 +319,30 @@ class JobScheduler:
                 preemption_default=preemption_default,
                 preemptible_rules=preemptible_rules,
             )
+        elif tes:
+            self._local_executor = CPUExecutor(
+                workflow,
+                dag,
+                local_cores,
+                printreason=printreason,
+                quiet=quiet,
+                printshellcmds=printshellcmds,
+                latency_wait=latency_wait,
+                cores=local_cores,
+                keepincomplete=keepincomplete,
+            )
+
+            self._executor = TaskExecutionServiceExecutor(
+                workflow,
+                dag,
+                cores=local_cores,
+                printreason=printreason,
+                quiet=quiet,
+                printshellcmds=printshellcmds,
+                latency_wait=latency_wait,
+                tes_url=tes,
+                container_image=container_image,
+            )
 
         else:
             self._executor = CPUExecutor(
@@ -312,6 +356,7 @@ class JobScheduler:
                 latency_wait=latency_wait,
                 cores=cores,
                 keepincomplete=keepincomplete,
+                keepmetadata=keepmetadata,
             )
         if self.max_jobs_per_second and not self.dryrun:
             max_jobs_frac = Fraction(self.max_jobs_per_second).limit_denominator()
@@ -323,10 +368,35 @@ class JobScheduler:
             # essentially no rate limit
             self.rate_limiter = DummyRateLimiter()
 
+        # Choose job selector (greedy or ILP)
+        self.job_selector = self.job_selector_greedy
+        if scheduler_type == "ilp":
+            import pulp
+
+            if pulp.apis.LpSolverDefault is None:
+                logger.warning(
+                    "Falling back to greedy scheduler because no default "
+                    "solver is found for pulp (you have to install either "
+                    "coincbc or glpk)."
+                )
+            else:
+                self.job_selector = self.job_selector_ilp
+
         self._user_kill = None
-        signal.signal(signal.SIGTERM, self.exit_gracefully)
+        try:
+            signal.signal(signal.SIGTERM, self.exit_gracefully)
+        except ValueError:
+            # If this fails, it is due to scheduler not being invoked in the main thread.
+            # This can only happen with --gui, in which case it is fine for now.
+            pass
         self._open_jobs.release()
 
+    def executor_error_callback(self, exception):
+        with self._lock:
+            self._executor_error = exception
+            # next scheduling round to catch and raise error
+            self._open_jobs.release()
+
     @property
     def stats(self):
         try:
@@ -334,21 +404,30 @@ class JobScheduler:
         except AttributeError:
             raise TypeError("Executor does not support stats")
 
-    def candidate(self, job):
-        """ Return whether a job is a candidate to be executed. """
-        return (
-            job not in self.running
-            and job not in self.failed
-            and (self.dryrun or (not job.dynamic_input and not self.dag.dynamic(job)))
-        )
-
     @property
     def open_jobs(self):
-        """ Return open jobs. """
-        return filter(self.candidate, list(job for job in self.dag.ready_jobs))
+        """Return open jobs."""
+        jobs = self.dag.ready_jobs
+
+        if not self.dryrun:
+            jobs = [
+                job
+                for job in jobs
+                if not job.dynamic_input and not self.dag.dynamic(job)
+            ]
+        return jobs
+
+    @property
+    def remaining_jobs(self):
+        """Return jobs to be scheduled including not yet ready ones."""
+        return [
+            job
+            for job in self.dag.needrun_jobs
+            if job not in self.running and not self.dag.finished(job)
+        ]
 
     def schedule(self):
-        """ Schedule jobs that are ready, maximizing cpu usage. """
+        """Schedule jobs that are ready, maximizing cpu usage."""
         try:
             while True:
                 # work around so that the wait does not prevent keyboard interrupts
@@ -358,19 +437,25 @@ class JobScheduler:
 
                 # obtain needrun and running jobs in a thread-safe way
                 with self._lock:
-                    needrun = list(self.open_jobs)
+                    self._finish_jobs()
+                    self._error_jobs()
+                    needrun = set(self.open_jobs)
                     running = list(self.running)
                     errors = self._errors
+                    executor_error = self._executor_error
                     user_kill = self._user_kill
 
                 # handle errors
-                if user_kill or (not self.keepgoing and errors):
+                if user_kill or (not self.keepgoing and errors) or executor_error:
                     if user_kill == "graceful":
                         logger.info(
                             "Will exit after finishing " "currently running jobs."
                         )
 
-                    if not running:
+                    if executor_error:
+                        print_exception(executor_error, self.workflow.linemaps)
+
+                    if executor_error or not running:
                         logger.info("Shutting down, this might take some time.")
                         self._executor.shutdown()
                         if not user_kill:
@@ -400,11 +485,12 @@ class JobScheduler:
                         "Ready jobs ({}):\n\t".format(len(needrun))
                         + "\n\t".join(map(str, needrun))
                     )
-                    run = (
-                        self.job_selector_greedy(needrun)
-                        if self.scheduler_type == "greedy"
-                        else self.job_selector_ilp(needrun)
-                    )
+
+                    if not self._last_job_selection_empty:
+                        logger.info("Select jobs to execute...")
+                    run = self.job_selector(needrun)
+                    self._last_job_selection_empty = not run
+
                     logger.debug(
                         "Selected jobs ({}):\n\t".format(len(run))
                         + "\n\t".join(map(str, run))
@@ -415,6 +501,13 @@ class JobScheduler:
                 # update running jobs
                 with self._lock:
                     self.running.update(run)
+                    # remove from ready_jobs
+                    self.dag.register_running(run)
+
+                # reset params and resources because they might contain TBDs
+                if not self.dryrun:
+                    for job in run:
+                        job.reset_params_and_resources()
 
                 # actually run jobs
                 local_runjobs = [job for job in run if job.is_local]
@@ -428,6 +521,42 @@ class JobScheduler:
             self._executor.cancel()
             return False
 
+    def _finish_jobs(self):
+        # must be called from within lock
+        for job in self._tofinish:
+            if self.handle_job_success:
+                try:
+                    self.get_executor(job).handle_job_success(job)
+                except (RuleException, WorkflowError) as e:
+                    # if an error occurs while processing job output,
+                    # we do the same as in case of errors during execution
+                    print_exception(e, self.workflow.linemaps)
+                    self._handle_error(job)
+                    return
+
+            if self.update_resources:
+                # normal jobs have len=1, group jobs have len>1
+                self.finished_jobs += len(job)
+                self.running.remove(job)
+                self._free_resources(job)
+
+            if self.print_progress:
+                if job.is_group():
+                    for j in job:
+                        logger.job_finished(jobid=j.jobid)
+                else:
+                    logger.job_finished(jobid=job.jobid)
+                self.progress()
+
+            self.dag.finish(job, update_dynamic=self.update_dynamic)
+        self._tofinish.clear()
+
+    def _error_jobs(self):
+        # must be called from within lock
+        for job in self._toerror:
+            self._handle_error(job)
+        self._toerror.clear()
+
     def run(self, jobs, executor=None):
         if executor is None:
             executor = self._executor
@@ -455,58 +584,25 @@ class JobScheduler:
     def _proceed(
         self,
         job,
-        update_dynamic=True,
-        print_progress=False,
-        update_resources=True,
-        handle_job_success=True,
     ):
-        """ Do stuff after job is finished. """
+        """Do stuff after job is finished."""
         with self._lock:
-            if handle_job_success:
-                # by calling this behind the lock, we avoid race conditions
-                try:
-                    self.get_executor(job).handle_job_success(job)
-                except (RuleException, WorkflowError) as e:
-                    # if an error occurs while processing job output,
-                    # we do the same as in case of errors during execution
-                    print_exception(e, self.workflow.linemaps)
-                    self._handle_error(job)
-                    return
+            self._tofinish.append(job)
 
-            try:
-                self.dag.finish(job, update_dynamic=update_dynamic)
-            except (RuleException, WorkflowError) as e:
-                # if an error occurs while processing job output,
-                # we do the same as in case of errors during execution
-                print_exception(e, self.workflow.linemaps)
-                self._handle_error(job)
-                return
-
-            if update_resources:
-                # normal jobs have len=1, group jobs have len>1
-                self.finished_jobs += len(job)
-                self.running.remove(job)
-                self._free_resources(job)
-
-            if print_progress:
-                if job.is_group():
-                    for j in job:
-                        logger.job_finished(jobid=j.jobid)
-                else:
-                    logger.job_finished(jobid=job.jobid)
-                self.progress()
-
-            if (
-                any(self.open_jobs)
-                or not self.running
-                or self.workflow.immediate_submit
-            ):
-                # go on scheduling if open jobs are ready or no job is running
+            if self.dryrun:
+                if len(self.running) - len(self._tofinish) - len(self._toerror) <= 0:
+                    # During dryrun, only release when all running jobs are done.
+                    # This saves a lot of time, as self.open_jobs has to be
+                    # evaluated less frequently.
+                    self._open_jobs.release()
+            else:
+                # go on scheduling if there is any free core
                 self._open_jobs.release()
 
     def _error(self, job):
         with self._lock:
-            self._handle_error(job)
+            self._toerror.append(job)
+            self._open_jobs.release()
 
     def _handle_error(self, job):
         """Clear jobs and stop the workflow.
@@ -523,12 +619,13 @@ class JobScheduler:
         if job.restart_times > job.attempt - 1:
             logger.info("Trying to restart job {}.".format(self.dag.jobid(job)))
             job.attempt += 1
+            # add job to those being ready again
+            self.dag._ready_jobs.add(job)
         else:
             self._errors = True
             self.failed.add(job)
             if self.keepgoing:
                 logger.info("Job failed, going on with independent jobs.")
-        self._open_jobs.release()
 
     def exit_gracefully(self, *args):
         with self._lock:
@@ -541,96 +638,180 @@ class JobScheduler:
         """
         import pulp
         from pulp import lpSum
+        from stopit import ThreadingTimeout as Timeout, TimeoutException
 
-        # assert self.resources["_cores"] > 0
-        scheduled_jobs = {
-            job: pulp.LpVariable(
-                "job_{job}_{idx}".format(job=job, idx=idx),
-                lowBound=0,
-                upBound=1,
-                cat=pulp.LpInteger,
+        if len(jobs) == 1:
+            logger.debug(
+                "Using greedy selector because only single job has to be scheduled."
             )
-            for idx, job in enumerate(jobs)
-        }
+            return self.job_selector_greedy(jobs)
 
-        temp_files = {
-            temp_file for job in jobs for temp_file in self.dag.temp_input(job)
-        }
+        with self._lock:
+            if not self.resources["_cores"]:
+                return set()
 
-        temp_job_improvement = {
-            temp_file: pulp.LpVariable(
-                temp_file, lowBound=0, upBound=1, cat="Continuous"
-            )
-            for temp_file in temp_files
-        }
-        prob = pulp.LpProblem("JobScheduler", pulp.LpMaximize)
+            # assert self.resources["_cores"] > 0
+            scheduled_jobs = {
+                job: pulp.LpVariable(
+                    "job_{}".format(idx),
+                    lowBound=0,
+                    upBound=1,
+                    cat=pulp.LpInteger,
+                )
+                for idx, job in enumerate(jobs)
+            }
+
+            def size_gb(f):
+                if self.touch:
+                    # In case of touch mode, there is no need to prioritize based on size.
+                    # We cannot access it anyway, because the files might be temporary and
+                    # not present.
+                    return 0
+                else:
+                    return f.size / 1e9
+
+            temp_files = {
+                temp_file for job in jobs for temp_file in self.dag.temp_input(job)
+            }
+
+            temp_job_improvement = {
+                temp_file: pulp.LpVariable(
+                    "temp_file_{}".format(idx), lowBound=0, upBound=1, cat="Continuous"
+                )
+                for idx, temp_file in enumerate(temp_files)
+            }
+
+            temp_file_deletable = {
+                temp_file: pulp.LpVariable(
+                    "deletable_{}".format(idx),
+                    lowBound=0,
+                    upBound=1,
+                    cat=pulp.LpInteger,
+                )
+                for idx, temp_file in enumerate(temp_files)
+            }
+            prob = pulp.LpProblem("JobScheduler", pulp.LpMaximize)
 
-        total_temp_size = max(sum([temp_file.size for temp_file in temp_files]), 1)
-        total_core_requirement = sum(
-            [job.resources.get("_cores", 1) + 1 for job in jobs]
-        )
-        # Objective function
-        # Job priority > Core load
-        # Core load > temp file removal
-        # Instant removal > temp size
-        # temp file size > fast removal?!
-        prob += (
-            total_core_requirement
-            * total_temp_size
-            * lpSum([job.priority * scheduled_jobs[job] for job in jobs])
-            + total_temp_size
-            * lpSum(
-                [
-                    (job.resources.get("_cores", 1) + 1) * scheduled_jobs[job]
-                    for job in jobs
-                ]
+            total_temp_size = max(
+                sum([size_gb(temp_file) for temp_file in temp_files]), 1
             )
-            + lpSum(
-                [
-                    temp_job_improvement[temp_file] * temp_file.size
-                    for temp_file in temp_files
-                ]
+            total_core_requirement = sum(
+                [max(job.resources.get("_cores", 1), 1) for job in jobs]
             )
-        )
-
-        # Constraints:
-        for name in self.workflow.global_resources:
+            # Objective function
+            # Job priority > Core load
+            # Core load > temp file removal
+            # Instant removal > temp size
             prob += (
-                lpSum(
-                    [scheduled_jobs[job] * job.resources.get(name, 0) for job in jobs]
+                2
+                * total_core_requirement
+                * 2
+                * total_temp_size
+                * lpSum([job.priority * scheduled_jobs[job] for job in jobs])
+                + 2
+                * total_temp_size
+                * lpSum(
+                    [
+                        max(job.resources.get("_cores", 1), 1) * scheduled_jobs[job]
+                        for job in jobs
+                    ]
+                )
+                + total_temp_size
+                * lpSum(
+                    [
+                        temp_file_deletable[temp_file] * size_gb(temp_file)
+                        for temp_file in temp_files
+                    ]
+                )
+                + lpSum(
+                    [
+                        temp_job_improvement[temp_file] * size_gb(temp_file)
+                        for temp_file in temp_files
+                    ]
                 )
-                <= self.resources[name],
-                "Limitation of resource: {name}".format(name=name),
             )
 
-        # Choose jobs that lead to "fastest" (minimum steps) removal of existing temp file
-        for temp_file in temp_files:
-            prob += temp_job_improvement[temp_file] <= lpSum(
-                [
-                    scheduled_jobs[job] * self.required_by_job(temp_file, job)
-                    for job in jobs
-                ]
-            ) / lpSum([self.required_by_job(temp_file, job) for job in jobs])
+            # Constraints:
+            for name in self.workflow.global_resources:
+                prob += (
+                    lpSum(
+                        [
+                            scheduled_jobs[job] * job.resources.get(name, 0)
+                            for job in jobs
+                        ]
+                    )
+                    <= self.resources[name]
+                )
 
-        # TODO enable this code once we have switched to pulp >=2.0
-        # if pulp.apis.LpSolverDefault is None:
-        #     raise WorkflowError(
-        #         "You need to install at least one LP solver compatible with PuLP (e.g. coincbc). "
-        #         "See https://coin-or.github.io/pulp for details. Alternatively, run Snakemake with "
-        #         "--scheduler greedy."
-        #     )
-        # # disable extensive logging
-        # pulp.apis.LpSolverDefault.msg = False
-        prob.solve()
-        selected_jobs = [
+            # Choose jobs that lead to "fastest" (minimum steps) removal of existing temp file
+            remaining_jobs = self.remaining_jobs
+            for temp_file in temp_files:
+                prob += temp_job_improvement[temp_file] <= lpSum(
+                    [
+                        scheduled_jobs[job] * self.required_by_job(temp_file, job)
+                        for job in jobs
+                    ]
+                ) / lpSum(
+                    [self.required_by_job(temp_file, job) for job in remaining_jobs]
+                )
+
+                prob += (
+                    temp_file_deletable[temp_file] <= temp_job_improvement[temp_file]
+                )
+
+        try:
+            with Timeout(10, swallow_exc=False):
+                self._solve_ilp(prob)
+        except TimeoutException as e:
+            logger.warning(
+                "Failed to solve scheduling problem with ILP solver in time (10s). "
+                "Falling back to greedy solver."
+            )
+            return self.job_selector_greedy(jobs)
+        except pulp.apis.core.PulpSolverError as e:
+            logger.warning(
+                "Failed to solve scheduling problem with ILP solver. Falling back to greedy solver. "
+                "Run Snakemake with --verbose to see the full solver output for debugging the problem."
+            )
+            return self.job_selector_greedy(jobs)
+
+        selected_jobs = set(
             job for job, variable in scheduled_jobs.items() if variable.value() == 1.0
-        ]
+        )
+
+        if not selected_jobs:
+            # No selected jobs. This could be due to insufficient resources or a failure in the ILP solver
+            # Hence, we silently fall back to the greedy solver to make sure that we don't miss anything.
+            return self.job_selector_greedy(jobs)
+
         for name in self.workflow.global_resources:
             self.resources[name] -= sum(
                 [job.resources.get(name, 0) for job in selected_jobs]
             )
         return selected_jobs
 
+    def _solve_ilp(self, prob):
+        import pulp
+
+        old_path = os.environ["PATH"]
+        if self.workflow.scheduler_solver_path is None:
+            # Temporarily prepend the given snakemake env to the path, such that the solver can be found in any case.
+            # This is needed for cluster envs, where the cluster job might have a different environment but
+            # still needs access to the solver binary.
+            os.environ["PATH"] = "{}:{}".format(
+                self.workflow.scheduler_solver_path, os.environ["PATH"]
+            )
+        try:
+            solver = (
+                pulp.get_solver(self.scheduler_ilp_solver)
+                if self.scheduler_ilp_solver
+                else pulp.apis.LpSolverDefault
+            )
+        finally:
+            os.environ["PATH"] = old_path
+        solver.msg = self.workflow.verbose
+        prob.solve(solver)
+
     def required_by_job(self, temp_file, job):
         return 1 if temp_file in self.dag.temp_input(job) else 0
 
@@ -644,6 +825,8 @@ class JobScheduler:
             jobs (list):    list of jobs
         """
         with self._lock:
+            if not self.resources["_cores"]:
+                return set()
             # each job is an item with one copy (0-1 MDKP)
             n = len(jobs)
             x = [0] * n  # selected jobs
@@ -695,7 +878,7 @@ class JobScheduler:
                 if not E:
                     break
 
-            solution = [job for job, sel in zip(jobs, x) if sel]
+            solution = set(job for job, sel in zip(jobs, x) if sel)
             # update resources
             for name, b_i in zip(self.global_resources, b):
                 self.resources[name] = b_i
@@ -732,8 +915,14 @@ class JobScheduler:
             temp_size = 0
             input_size = 0
         else:
-            temp_size = self.dag.temp_size(job)
-            input_size = job.inputsize
+            try:
+                temp_size = self.dag.temp_size(job)
+                input_size = job.inputsize
+            except FileNotFoundError:
+                # If the file is not yet present, this shall not affect the
+                # job selection.
+                temp_size = 0
+                input_size = 0
 
         # Usually, this should guide the scheduler to first schedule all jobs
         # that remove the largest temp file, then the second largest and so on.
@@ -746,5 +935,5 @@ class JobScheduler:
         return (job.priority, temp_size, input_size)
 
     def progress(self):
-        """ Display the progress. """
+        """Display the progress."""
         logger.progress(done=self.finished_jobs, total=len(self.dag))
