@@ -1,6 +1,6 @@
 __author__ = "Johannes Köster"
-__copyright__ = "Copyright 2015-2019, Johannes Köster"
-__email__ = "koester@jimmy.harvard.edu"
+__copyright__ = "Copyright 2021, Johannes Köster"
+__email__ = "johannes.koester@uni-due.de"
 __license__ = "MIT"
 
 import _io
@@ -9,15 +9,16 @@ import os
 import subprocess as sp
 import inspect
 import shutil
+import stat
+import tempfile
 import threading
 
-from snakemake.utils import format, argvquote, _find_bash_on_windows
-from snakemake.common import ON_WINDOWS
+from snakemake.utils import format, argvquote, cmd_exe_quote, find_bash_on_windows
+from snakemake.common import ON_WINDOWS, RULEFUNC_CONTEXT_MARKER
 from snakemake.logging import logger
 from snakemake.deployment import singularity
 from snakemake.deployment.conda import Conda
 from snakemake.exceptions import WorkflowError
-import snakemake
 
 
 __author__ = "Johannes Köster"
@@ -29,6 +30,14 @@ if not isinstance(sys.stdout, _io.TextIOWrapper):
     STDOUT = None
 
 
+# There is a max length for a command executed as well as a maximum
+# length for each argument passed to a command. The latter impacts us
+# especially when doing `sh -c 'long script from user'`. On Linux, it's
+# hardcoded in the kernel as 32 pages, or 128kB. On OSX it appears to be
+# close to `getconf ARG_MAX`, about 253kb.
+MAX_ARG_LEN = 16 * 4096 - 1
+
+
 class shell:
     _process_args = {}
     _process_prefix = ""
@@ -36,6 +45,7 @@ class shell:
     _lock = threading.Lock()
     _processes = {}
     _win_command_prefix = ""
+    conda_block_conflicting_envvars = True
 
     @classmethod
     def get_executable(cls):
@@ -54,7 +64,7 @@ class shell:
 
     @classmethod
     def executable(cls, cmd):
-        if os.name in ("posix", "nt") and not os.path.isabs(cmd):
+        if cmd and not os.path.isabs(cmd):
             # always enforce absolute path
             cmd = shutil.which(cmd)
             if not cmd:
@@ -63,9 +73,20 @@ class shell:
                     "is not available in your "
                     "PATH.".format(cmd)
                 )
-        if os.path.split(cmd)[-1].lower() in ("bash", "bash.exe"):
+        if ON_WINDOWS:
+            if cmd is None:
+                cls._process_prefix = ""
+                cls._win_command_prefix = ""
+            elif os.path.split(cmd)[-1].lower() in ("bash", "bash.exe"):
+                if cmd == r"C:\Windows\System32\bash.exe":
+                    raise WorkflowError(
+                        "Cannot use WSL bash.exe on Windows. Ensure that you have "
+                        "a usable bash.exe availble on your path."
+                    )
+                cls._process_prefix = "set -euo pipefail; "
+                cls._win_command_prefix = "-c"
+        elif os.path.split(cmd)[-1].lower() == "bash":
             cls._process_prefix = "set -euo pipefail; "
-            cls._win_command_prefix = "-c"
         cls._process_args["executable"] = cmd
 
     @classmethod
@@ -79,7 +100,7 @@ class shell:
     @classmethod
     def win_command_prefix(cls, cmd):
         """The command prefix used on windows when specifing a explicit
-        shell executable. This would be "-c" for bash and "/C" for cmd.exe
+        shell executable. This would be "-c" for bash.
         Note: that if no explicit executable is set commands are executed
         with Popen(..., shell=True) which uses COMSPEC on windows where this
         is not needed.
@@ -103,54 +124,88 @@ class shell:
     ):
         if "stepout" in kwargs:
             raise KeyError("Argument stepout is not allowed in shell command.")
+
+        if ON_WINDOWS and not cls.get_executable():
+            # If bash is not used on Windows quoting must be handled in a special way
+            kwargs["quote_func"] = cmd_exe_quote
+
         cmd = format(cmd, *args, stepout=2, **kwargs)
-        context = inspect.currentframe().f_back.f_locals
-        # add kwargs to context (overwriting the locals of the caller)
-        context.update(kwargs)
 
         stdout = sp.PIPE if iterable or read else STDOUT
 
         close_fds = sys.platform != "win32"
 
+        func_context = inspect.currentframe().f_back.f_locals
+
+        if func_context.get(RULEFUNC_CONTEXT_MARKER):
+            # If this comes from a rule, we expect certain information to be passed
+            # implicitly via the rule func context, which is added here.
+            context = func_context
+        else:
+            # Otherwise, context is just filled via kwargs.
+            context = dict()
+        # add kwargs to context (overwriting the locals of the caller)
+        context.update(kwargs)
+
         jobid = context.get("jobid")
         if not context.get("is_shell"):
             logger.shellcmd(cmd)
 
-        env_prefix = ""
         conda_env = context.get("conda_env", None)
+        conda_base_path = context.get("conda_base_path", None)
         container_img = context.get("container_img", None)
         env_modules = context.get("env_modules", None)
         shadow_dir = context.get("shadow_dir", None)
+        resources = context.get("resources", {})
+        singularity_args = context.get("singularity_args", "")
+        threads = context.get("threads", 1)
 
-        cmd = "{} {} {}".format(
-            cls._process_prefix, cmd.strip(), cls._process_suffix
-        ).strip()
+        cmd = " ".join((cls._process_prefix, cmd, cls._process_suffix)).strip()
 
         if env_modules:
             cmd = env_modules.shellcmd(cmd)
             logger.info("Activating environment modules: {}".format(env_modules))
 
         if conda_env:
-            cmd = Conda(container_img).shellcmd(conda_env, cmd)
+            if ON_WINDOWS and not cls.get_executable():
+                # If we use cmd.exe directly on winodws we need to prepend batch activation script.
+                cmd = Conda(container_img, prefix_path=conda_base_path).shellcmd_win(
+                    conda_env, cmd
+                )
+            else:
+                cmd = Conda(container_img, prefix_path=conda_base_path).shellcmd(
+                    conda_env, cmd
+                )
+
+        tmpdir = None
+        if len(cmd.replace("'", r"'\''")) + 2 > MAX_ARG_LEN:
+            tmpdir = tempfile.mkdtemp(dir=".snakemake", prefix="shell_tmp.")
+            script = os.path.join(os.path.abspath(tmpdir), "script.sh")
+            with open(script, "w") as script_fd:
+                print(cmd, file=script_fd)
+            os.chmod(script, os.stat(script).st_mode | stat.S_IXUSR | stat.S_IRUSR)
+            cmd = '"{}" "{}"'.format(cls.get_executable() or "/bin/sh", script)
 
         if container_img:
-            args = context.get("singularity_args", "")
             cmd = singularity.shellcmd(
                 container_img,
                 cmd,
-                args,
+                singularity_args,
+                envvars=None,
                 shell_executable=cls._process_args["executable"],
                 container_workdir=shadow_dir,
+                is_python_script=context.get("is_python_script", False),
             )
             logger.info("Activating singularity image {}".format(container_img))
         if conda_env:
             logger.info("Activating conda environment: {}".format(conda_env))
 
-        threads = str(context.get("threads", 1))
+        tmpdir_resource = resources.get("tmpdir", None)
         # environment variable lists for linear algebra libraries taken from:
         # https://stackoverflow.com/a/53224849/2352071
         # https://github.com/xianyi/OpenBLAS/tree/59243d49ab8e958bb3872f16a7c0ef8c04067c0a#setting-the-number-of-threads-using-environment-variables
         envvars = dict(os.environ)
+        threads = str(threads)
         envvars["OMP_NUM_THREADS"] = threads
         envvars["GOTO_NUM_THREADS"] = threads
         envvars["OPENBLAS_NUM_THREADS"] = threads
@@ -158,8 +213,32 @@ class shell:
         envvars["VECLIB_MAXIMUM_THREADS"] = threads
         envvars["NUMEXPR_NUM_THREADS"] = threads
 
-        use_shell = True
+        if tmpdir_resource:
+            envvars["TMPDIR"] = tmpdir_resource
+            envvars["TMP"] = tmpdir_resource
+            envvars["TEMPDIR"] = tmpdir_resource
+            envvars["TEMP"] = tmpdir_resource
 
+        if "additional_envvars" in kwargs:
+            env = kwargs["additional_envvars"]
+            if not isinstance(env, dict) or not all(
+                isinstance(v, str) for v in env.values()
+            ):
+                raise WorkflowError(
+                    "Given environment variables for shell command have to be a dict of strings, "
+                    "but the following was provided instead:\n{}".format(env)
+                )
+            envvars.update(env)
+
+        if conda_env and cls.conda_block_conflicting_envvars:
+            # remove envvars that conflict with conda
+            for var in ["R_LIBS", "PYTHONPATH", "PERLLIB", "PERL5LIB"]:
+                try:
+                    del envvars[var]
+                except KeyError:
+                    pass
+
+        use_shell = True
         if ON_WINDOWS and cls.get_executable():
             # If executable is set on Windows shell mode can not be used
             # and the executable should be prepended the command together
@@ -186,7 +265,7 @@ class shell:
 
         ret = None
         if iterable:
-            return cls.iter_stdout(proc, cmd)
+            return cls.iter_stdout(proc, cmd, tmpdir)
         if read:
             ret = proc.stdout.read()
         if bench_record is not None:
@@ -197,6 +276,9 @@ class shell:
         else:
             retcode = proc.wait()
 
+        if tmpdir:
+            shutil.rmtree(tmpdir)
+
         if jobid is not None:
             with cls._lock:
                 del cls._processes[jobid]
@@ -206,10 +288,12 @@ class shell:
         return ret
 
     @staticmethod
-    def iter_stdout(proc, cmd):
+    def iter_stdout(proc, cmd, tmpdir):
         for l in proc.stdout:
             yield l[:-1]
         retcode = proc.wait()
+        if tmpdir:
+            shutil.rmtree(tmpdir)
         if retcode:
             raise sp.CalledProcessError(retcode, cmd)
 
@@ -231,3 +315,5 @@ if os.name == "posix":
             shell.executable("sh")
     else:
         shell.executable("bash")
+elif ON_WINDOWS:
+    shell.executable(None)
